/***************************************************************************
                          myldapklient.cpp  -  description
                             -------------------
    begin                : Mon Sep  3 19:35:50 CEST 2001
    copyright            : (C) 2001 by fsn
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "myldapklient.h"

// Should probably use the KIO_LDAP. However, I seem to get bad data with it, so
// therefore I use openldap 2.0.x directly instead

//---------------------------------------------------------------------
// Constructor and initialization

MyldapKlient::MyldapKlient(QWidget *parent, const char *name) : ldapdlg(parent, name)
{

  KPasswordDialog::disableCoreDumps();
	init();

}

//---------------------------------------------------------------------
// Destructor.

MyldapKlient::~MyldapKlient()
{
	config->sync();
	delete config;
}

//---------------------------------------------------------------------
// Do some simple inits.

void MyldapKlient::init() {
	QString server;
  int id;

  // ListView popup menu setup

  popUp = new KPopupMenu (ldapView, "Action");
  id = popUp->insertItem ( "Add to global addressbook");
  popUp->connectItem ( id, this, SLOT(slot_addToKab()) );
  id = popUp->insertItem ( "Copy to clipboard");
  popUp->connectItem ( id, this, SLOT(slot_toClipboard()) );
  id = popUp->insertItem ( "Send mail to...");
  popUp->connectItem ( id, this, SLOT(slot_mailto()) );
  connect(ldapView, SIGNAL( contextMenuRequested( QListViewItem *, const QPoint& , int ) ),
    this, SLOT( slot_RMB( QListViewItem *, const QPoint &, int ) ) ); // Magic


	// Initial setup

	config = new KSimpleConfig("myldapklientrc",false);
	if (!config->hasGroup("Global")) {
		KMessageBox::information(this,i18n("You have no configuration file.\nClick on Configure in the main dialog."));
		return;		
	};
	config->setGroup("Global");
	if ( (config->hasKey("lastserver")) && (config->readEntry("lastserver") != "") ) {
		server=config->readEntry("lastserver");
	} else {
		if (config->hasGroup("Servers")) {
			config->setGroup("Servers");
			server=config->readEntry("server0");
		} else {
			server="";
		}
	}

	buildServerComboBox(server);
  newServer(server);
}


//---------------------------------------------------------------------
// Run this when we change, or set, the server.
// Run from slot_serverChanged() and initLists().

void MyldapKlient::newServer(QString server) {

	QString group,key,message;
	int i=0;	

  knownPass = "";
	schemeList.setAutoDelete( TRUE );
	kabentryList.setAutoDelete( TRUE );
	schemeList.clear();
	kabentryList.clear();

	// Clear the listview from columns and info

	for (i= this->ldapView->columns(); i>0; i--) {
		this->ldapView->removeColumn(i-1);
	}
	this->ldapView->clear();                	// Unclear if we need this.

	if (server == "") {return;};							// If server is blank, die quietly (misconfigured).

	// Get the chosen server

	group="Server"+server;
	if (!config->hasGroup(group)) {
		KMessageBox::error(this,i18n("Server "+server+" has no config group in config file."));
		return;
	};
	config->setGroup(group);


	// This part builds the two lists. It's not pretty, but it should work, as long as you have
	// complete triplets of ldapattrN and kabtagN and headN in the config. If things get
  // messed up, well...	

	for (i=0; i<LAMEMAX; i++) { 			// Lame. Should terminate when exhausted, and not max out at LAMEMAX.

		// Add those listview columns.
		key="";
    QTextOStream(&key) << "head" << i;
		if (config->hasKey(key)) {
			this->ldapView->addColumn( config->readEntry(key) , -1);
		} else {
			break;
		}

		// Build the LDAP scheme list
		key="";
		QTextOStream(&key) << "ldapattr" << i;
		if (config->hasKey(key)) {
			schemeList.append( new QString(config->readEntry(key)) );
			// cout << "scheme "<<key<<":"<<config->readEntry(key)<<"\n";
		} else {
			break;
		}

		// Build the Kabtag mapping list
		key="";
		QTextOStream(&key) << "kabtag" << i;
		if (config->hasKey(key)) {
			kabentryList.append( new QString(config->readEntry(key)) );
			// cout << "kabentry "<<key<<":"<<config->readEntry(key)<<"\n";
		} else {
			break;
		}

	}
	// The lists are now finished

	buildAllSearchAttrCombo(server);
	this->search0->setFocus();
}

//---------------------------------------------------------------------
// Build the server comboBox entries from the [Servers] config group

void MyldapKlient::buildServerComboBox(QString & inServer) {
	QString key,thisServer;
	int i;

	this->ldapServSel->clear();

	if (!config->hasGroup("Servers")) {
		KMessageBox::error(this,i18n("No list of LDAP servers in configfile.\nClick on Configure in the main dialog."));
		return;
	};

	config->setGroup("Servers");
	for (i=0; i<LAMEMAX; i++) { 			// Lame. Should terminate when exhausted, and not max out at LAMEMAX.
		key="";
		QTextOStream(&key) << "server" << i;
		if (config->hasKey(key)) {
			thisServer=config->readEntry(key);
			this->ldapServSel->insertItem( thisServer , -1);
			if (thisServer == inServer) {this->ldapServSel->setCurrentItem(i); };
		} else {
			break;
		}
	}
}

//---------------------------------------------------------------------
// Build a search ComboBox. We are building these Combo boxes from the nicelooking
// headings we defined. Not that order in all the three schemelists is important, in
// head, ldapattr and kabtag. I promise it all makes perfect sense.

void MyldapKlient::buildSearchAttrCombo(QComboBox * inCombo, QString & inServer, int selected) {
	QString key,thisServer,heading;
	int i;
	
	inCombo->clear();
	inCombo->insertItem("<Search>",-1);
	
	thisServer="";
	QTextOStream(&thisServer) << "Server" << inServer;
	if (!config->hasGroup(thisServer)) { return; };
	config->setGroup(thisServer);
	for (i=0; i<LAMEMAX; i++) { 			// Lame.
		key="";
		QTextOStream(&key) << "head" << i;
		if (config->hasKey(key)) {
			heading=config->readEntry(key);
			inCombo->insertItem( heading , -1);
		} else {
			break;
		}
	}
	if (selected > 0) { inCombo->setCurrentItem(selected) ; }
}

//---------------------------------------------------------------------
// Build all search ComboBoxes

void MyldapKlient::buildAllSearchAttrCombo(QString & inServer) {
	QString group,key,search;
	int selected;

	group="Server"+inServer;
	if (!config->hasGroup(group)) { return; };
	config->setGroup(group);
	key="";		

	selected=config->readEntry("search0").toInt();
	buildSearchAttrCombo(this->searchAttr0 , inServer, selected);
	this->search0->clear();
	
	selected=config->readEntry("search1").toInt();
	buildSearchAttrCombo(this->searchAttr1 , inServer, selected);
	this->search1->clear();

	selected=config->readEntry("search2").toInt();
	buildSearchAttrCombo(this->searchAttr2 , inServer, selected);
	this->search2->clear();
	
}

//---------------------------------------------------------------------
// Get the column number for an LDAP attribute. With a loop. Room for improvement.

int MyldapKlient::getSchemePos(QString & thisScheme) {
	QListIterator<QString> sit(schemeList);
	int i=0;
	bool flag=false;

	for ( ; sit.current(); ++sit ) {
  	if (*(sit.current()) == thisScheme ) { flag=true; break; }
		i++;
  }
	if (flag == true) {
		// cout << "Found " << thisScheme << " at " << i << ".\n";
		return i;
	} else {
		// cout << "Didn't find " << thisScheme << ".\n";
		return -1;
	}
	// cout << "Shouldn't ever get here.\n";
	return 0;
}


//---------------------------------------------------------------------
// Get the KabentryTag for a numerically given column

QString MyldapKlient::getKabentryTag(int thisCol) {
	QListIterator<QString> kit(kabentryList);

	int i=0;

	while (i<thisCol) {
		++kit;
		++i;
	}

	return *(kit.current());
}

//---------------------------------------------------------------------
// Get the SchemeAttr for a numerically given column

QString MyldapKlient::getSchemeAttr(int thisAttr) {
	QListIterator<QString> sit(schemeList);

	int i=0;

	while (i<thisAttr) {
		++sit;
		++i;
	}

	return *(sit.current());
}


//---------------------------------------------------------------------
// The server combobox changed, rebuild the listview and everything

void MyldapKlient::slot_serverChanged() {
	QString server;
	server = this->ldapServSel->currentText();
	newServer(server);
}

//---------------------------------------------------------------------
// User want's to search

void MyldapKlient::slot_doLdapSearch() {

	// Mostly for LDAP

	LDAP *ld;
	BerElement *ptr;
	char *dn, *attr,**vals;
	int result,rc,i,num,col;
	LDAPMessage *res,*e;

  // Other stuff

	QListViewItem *dLvi;

	// Some QStrings

	QString lAttr, lVal, ldapHost, mainFilter, ldapBase, server, group, temp;
	QString ldapDN,ldapPassword, ldapVersion, ldapFilter;

	// Config

	server = this->ldapServSel->currentText();

	if (server == NULL) { 								// Nothing. Unconfigured or something.
		KMessageBox::error(this,i18n("No server selected/configured."));
		return;
	}

	group="Server"+server;
	config->setGroup(group);

	ldapHost=config->readEntry("ldaphost");
	if ( (ldapHost == NULL) || (ldapHost == "") ) { 									// Nothing. Unconfigured or something.
		KMessageBox::error(this,i18n("No ldaphost configured for selected LDAP server."));
		return;
	}

	ldapBase=config->readEntry("ldapbase");
	if (ldapBase == NULL) { 									// Nothing. Unconfigured or something.
		KMessageBox::error(this,i18n("No ldapbase configured for selected LDAP server."));
		return;
	}

  ldapDN=config->readEntry("dn");
  ldapPassword=config->readEntry("password");
  ldapVersion=config->readEntry("ldapversion");
  ldapFilter=config->readEntry("ldapfilter");

  // Here is a simple handling of passwords

  if ( (ldapDN != "") && (ldapPassword == "") && (knownPass == "") ) {
    int result = KPasswordDialog::getPassword(knownPass, i18n("Password for "+ldapDN+" at "+ldapHost));
    if (result == KPasswordDialog::Accepted) {
      ldapPassword=knownPass;
    } else {
      return;
    }
  }

	// Write the position of the searchcomboboxes. config is setgrouped to the right server above.
	// I hope.

	temp.setNum(this->searchAttr0->currentItem());
	config->writeEntry("search0",temp );
	temp.setNum(this->searchAttr1->currentItem());
	config->writeEntry("search1",temp );
	temp.setNum(this->searchAttr2->currentItem());
	config->writeEntry("search2",temp );


	// Clear the view. All the old listviewentryobjects are supposed to be deleted now,
	// at least if I understrand the QTdocumentation correctly. Otherwise, here is a memory leak.

	this->ldapView->clear();

	// Build the filter from what's in the GUI textline
	// I really, really, really like the QTextOStream. No, I don't anymore.
	//
	// You can now actually search on one, two or three attributes.
  // Interface hall of shame mentions this kind of searching, with a fixed number of
  // search elements... Oh well.

	if ((this->search0->text() == "")&&(this->search1->text() == "")&&(this->search2->text() == "")) { 									// Nothing. Unconfigured or something.
		KMessageBox::error(this,i18n("No search filter entered."));
		return;
	}

	if ((this->searchAttr0->currentItem() == 0)&&(this->searchAttr1->currentItem() == 0)&&
			(this->searchAttr2->currentItem() == 0)) {
		KMessageBox::error(this,i18n("No search filter entered."));
		return;
	}

	// Were building a filterstring

  // IMPLEMENTATION NOTE: The user sets any searchAttrN to an item, thus giving us a number
  // for that item. That number corresponds to an item in the ldap attribute list, aptly named
  // schemeList. So, getSchemeAttr gives us an ldap attribute in a Qstring from the list
  // of attributes, based on what item the user selected in searchAttrN. Clear as crystal.
  // But it works.

	mainFilter = "(&";

	if ((this->search0->text() != "")&&(this->searchAttr0->currentItem() != 0)) {
		QTextOStream(&mainFilter)<< mainFilter << "("<<getSchemeAttr( (this->searchAttr0->currentItem( ))-1)<<"=*"<<
			this->search0->text()<<"*)";
	}		

	if ((this->search1->text() != "")&&(this->searchAttr1->currentItem() != 0)) {
		QTextOStream(&mainFilter)<< mainFilter << "("<<getSchemeAttr( (this->searchAttr1->currentItem( ))-1)<<"=*"<<
			this->search1->text()<<"*)";
	}		

	if ((this->search2->text() != "")&&(this->searchAttr2->currentItem() != 0)) {
		QTextOStream(&mainFilter)<< mainFilter << "("<<getSchemeAttr( (this->searchAttr2->currentItem( ))-1)<<"=*"<<
			this->search2->text()<<"*)";
	}		

	QTextOStream(&mainFilter)<< mainFilter <<  ldapFilter << ")";

	// Remember what we do

	config->setGroup("Global");
	config->writeEntry("lastserver",server);

	config->sync();		// Glurg. What's that sync doing there? I should only sync when
										// I'm exiting... Oh, well...

	// Do the LDAP init cha-cha.

	ld = ldap_init(ldapHost.latin1(), LDAP_PORT);
	int version = ldapVersion.toInt();
	result = ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &version);

  if ( ldapDN != "") {  // Really, this distinction is redundant, since
                        // the QStrings are initialized with empty strings.
                        // But, better safe than sorry, no?
    // With username and password
    result = ldap_simple_bind_s(ld,ldapDN.latin1(),ldapPassword.latin1());
  } else {
    // Anonymous
    result = ldap_simple_bind_s(ld,"","");

  }


	// Long timeouts here somewhere? That's because your client can't do reverse
	// lookup on the LDAP server IPaddress. And I do make a synchronus request,
	// so I won't get past the ldap_bind until the session really is initiated.
  // FIXME: Make the bind request timeout gracefully.

	if ( result == LDAP_SUCCESS) {

		// Some of the LDAP data handling code is stolen from another LDAP client project, gq-0.40, and then
		// stripped and modified.
			
		result = ldap_search_s(ld, ldapBase.latin1(), LDAP_SCOPE_SUBTREE, mainFilter.latin1(), NULL, 0,&res);
		num=0;
		if ( result == LDAP_SUCCESS ) {
			rc=1;
			while(rc) {

				// Basically the next block of lines was very stolen from gq. Thanx to you! GQ is also GPL, by the way.
				// They seem to know when and where to free memory. I don't worry about that
				// stuff....

				for(e = ldap_first_entry(ld, res); e != NULL; e = ldap_next_entry(ld,e)) {
					num++;
					dn=ldap_get_dn(ld,e);					
					ldap_memfree(dn);

					dLvi = new QListViewItem (this->ldapView);		// My line though

					for(attr = ldap_first_attribute(ld, e, &ptr); attr != NULL; attr = ldap_next_attribute(ld, e, ptr)) {
						lAttr = attr;
						vals = ldap_get_values(ld, e, attr);
						if (vals){
							for(i = 0; vals[i] != NULL; i++) {    		// Here is a potential for handling multiple
															                        	// values for the same attribute.
								lVal=QString::fromUtf8(vals[i]);
							}

							// Theft ends here.

							// Ok, I've patched everything up so that the LDAP scheme decoding is now a little bit
							// more dynamic.
							//
							// You should set an LDAP attributelist in the request so you don't recieve more
							// attributes than you need. But this works for now.

  // IMPLEMENTATION NOTE: Ok, we now have an ldap attribute in lAttr, and it's data in lVal.
  // So we take lAttr and checks it against the ldap attribute list, the one named schemeList,
  // and that gives us a number, which is the attributes position in that list. We now insert
  // the data in the column in the listview, that has the number that we got from the
  // attribute list. (Got that?) Thus, the listview is the carrier of all the data, it exists in
  // the listview only. When we click on Add to KAB or Mail to... we fetch the actual data from
  // the listview.

  //

							col = getSchemePos( lAttr );
							if (col>-1) { dLvi->setText(col,lVal); }	// Add the text to the listview

							ldap_value_free(vals);
						}
					}
				}			/* ??? */
				rc=0;
			}				/* ??? */
		} else {
      // If something goes wrong with the search request, we end up here. F.ex. if you've a malformed
      // extra LDAP filter.
      temp=ldap_err2string(result);
      switch ( result ) {
        case LDAP_FILTER_ERROR :
          KMessageBox::error(this,i18n("Malformed filter. Please check the syntax\n of the LDAP filter."));
          break;
  		  default :
          KMessageBox::error(this,i18n("LDAP request failed. LDAP error message:\n"+temp));
          break;
      }
    }
	} else {
    temp=ldap_err2string(result);
    switch ( result ) {
      case LDAP_INVALID_CREDENTIALS :
        knownPass = "";
        KMessageBox::error(this,i18n("Wrong LDAP password or DN."));
        break;
      case LDAP_SERVER_DOWN :
        KMessageBox::error(this,i18n("Could not bind to LDAP server."));
        break;
      default :
    		KMessageBox::error(this,i18n("LDAP bind failed. LDAP error message:\n"+temp));
        break;
    }
	}
	ldap_unbind(ld);		// Elvis has left the building
}

//---------------------------------------------------------------------
// Configure - let's fire up that configuration dialog - which is not here but in myconfig.h|cpp

void MyldapKlient::slot_configure() {
	myconfig *MyKonfig;

	MyKonfig = new myconfig(this,"Config",true);
	MyKonfig->exec();

	config->reparseConfiguration();	
	init();

}

//---------------------------------------------------------------------
// User has clicked on Add to KAB, so we'd better give him what he want's

void MyldapKlient::slot_addToKab() {

  QListViewItem *selItem;

  KABC::Addressee entry;
  KABC::PhoneNumber phoneHome;
  KABC::PhoneNumber phoneMobile;
  KABC::PhoneNumber phoneFax;
  KABC::PhoneNumber phoneModem;
  KABC::Address addressHome;
  KABC::Address addressWork;
  KABC::AddressBook *AddressBook;

  int i;
	QString tag,data;
	bool workFlag, homeFlag;

// ===

  phoneHome.setType(1);
  phoneMobile.setType(64);
  phoneFax.setType(32);
  phoneModem.setType(512);
  addressWork.setType(32);
  addressHome.setType(16);

  AddressBook = KABC::StdAddressBook::self();

	workFlag=FALSE;
	homeFlag=FALSE;

	// Shortcut. Really not needed, but nice.
	// Whew, selItem is a pointer. For a while I thought I was doing many copies...
	
	selItem = this->ldapView->selectedItem();

	if ( selItem == NULL ) { return; };
	
	// Umm... Not beautiful, but does it's job.

	for (i= this->ldapView->columns(); i>0; i--) {
		tag = getKabentryTag(i-1);
		data = selItem->text(i-1);

  // Simple matching. I mean, we could have infinite possibilities here,
  // but that would be harder. Perhaps later. Note to self: switch, perhaps?

		if ( tag == "emails" ) 							{entry.insertEmail(data,false); }
		if ( tag == "fn" ) 									{entry.setFormattedName ( data ) ; }
		if ( tag == "user1" ) 							{entry.insertCustom ("myldapklient","user1",data) ; }
		if ( tag == "user2" ) 							{entry.insertCustom ("myldapklient","user2",data) ; }
		if ( tag == "user3" ) 							{entry.insertCustom ("myldapklient","user3",data) ; }
		if ( tag == "user4" ) 							{entry.insertCustom ("myldapklient","user4",data) ; }
		if ( tag == "firstname") 						{entry.setGivenName( data ); }
		if ( tag == "middlename")						{entry.setAdditionalName ( data) ; }
		if ( tag == "lastname") 						{entry.setFamilyName ( data ) ; }
		if ( tag == "comment")							{entry.setNote ( data ) ; }
		if ( tag == "title") 								{entry.setTitle ( data ) ; }
		if ( tag == "rank")									{entry.insertCustom ("myldapklient","rank",data) ; }
//
		if ( tag == "phone/Fixed")					{phoneHome.setNumber(data);
																				 entry.insertPhoneNumber(phoneHome); }
		if ( tag == "phone/Mobile")					{phoneMobile.setNumber(data);
																				 entry.insertPhoneNumber(phoneMobile); }
		if ( tag == "phone/Fax")						{phoneFax.setNumber(data);
																				 entry.insertPhoneNumber(phoneFax); }
		if ( tag == "phone/Modem")					{phoneModem.setNumber(data);
																			   entry.insertPhoneNumber(phoneModem); }

		if ( tag == "Work/org")							{entry.setOrganization (data); }
		if ( tag == "Home/org")							{entry.insertCustom ("myldapklient","home/org",data); }

		if ( tag == "Work/orgUnit")					{entry.insertCustom ("myldapklient","work/orgUnit",data) ; }
		if ( tag == "Home/orgUnit")					{entry.insertCustom ("myldapklient","home/orgUnit",data) ; }
		if ( tag == "Work/orgSubUnit")			{entry.insertCustom ("myldapklient","work/orgSubUnit",data) ; }
		if ( tag == "Home/orgSubUnit")			{entry.insertCustom ("myldapklient","home/orgSubUnit",data) ; }

		if ( tag == "Work/country")  				{addressWork.setCountry (data); workFlag=TRUE; }
		if ( tag == "Home/country")			    {addressHome.setCountry(data); homeFlag=TRUE; }
		if ( tag == "Work/town")            {addressWork.setLocality(data); workFlag=TRUE; }
		if ( tag == "Home/town")						{addressHome.setLocality(data); homeFlag=TRUE; }
		if ( tag == "Work/state")						{addressWork.setRegion(data); workFlag=TRUE; }
		if ( tag == "Home/state")						{addressHome.setRegion(data); homeFlag=TRUE; }
		if ( tag == "Work/zip")		  				{addressWork.setPostalCode(data); workFlag=TRUE; }
		if ( tag == "Home/zip")		 					{addressHome.setPostalCode(data); homeFlag=TRUE; }
		if ( tag == "Work/address")					{addressWork.setStreet(data); workFlag=TRUE; }
		if ( tag == "Home/address")					{addressHome.setStreet(data); homeFlag=TRUE; }

		if ( tag == "Work/position")				{entry.insertCustom ("myldapklient","work/position",data); }
		if ( tag == "Home/position")				{entry.insertCustom ("myldapklient","home/position",data); }
		if ( tag == "Work/deliveryLabel") 	{entry.insertCustom ("myldapklient","work/deliveryLabel",data); }
		if ( tag == "Home/deliveryLabel") 	{entry.insertCustom ("myldapklient","home/deliveryLabel",data); }

		// We could do more matching here if we wanted. Mail me with your
		// favourite ldap attributes.

	}

	if (workFlag == TRUE) { entry.insertAddress(addressWork);};
	if (homeFlag == TRUE) { entry.insertAddress(addressHome);};

	// Add entry to stdaddressbook

	AddressBook->insertAddressee(entry);

	// .. and save. Cornelius, kabc IS really easier to work with than
	// kab/kab2. Great work!

	if ( !KABC::StdAddressBook::save() ) {
		KMessageBox::error( this, i18n("Couldn't save addressbook") );
	} else {
		KMessageBox::information(this,i18n("Entry successfully added to Adressbook."));
	}

}

//---------------------------------------------------------------------
// User has clicked on Mailto. Let's try and fire up kmail

void MyldapKlient::slot_mailto() {
	KShellProcess proc;
	QListViewItem *selItem;
	int i;
	QString tag,data,mailer;

	// Shortcut. Really not needed, but nice.
	
	selItem = this->ldapView->selectedItem();

	// Scan through the columns untill we find the correct kabentrytag

	if ( selItem == NULL ) { return; };
	for (i= this->ldapView->columns(); i>0; i--) {
		tag = getKabentryTag(i-1);
		data = selItem->text(i-1);
		if (tag == "emails") 	{break;}
	}

  // We can actually now configure which mailer to use. Wo-hoo...

	config->setGroup("Global");
	if (config->hasKey("mailer") ) {
		mailer=config->readEntry("mailer");			// Get the configured mailer, or....
	} else {
		mailer="kmail";                         // use a default, kmail ofcourse.
		config->writeEntry("mailer",mailer );   // Next time, it'll be in the config
		config->sync();			// I don't think this one should be here...
	}

	// Configure 'proc' to start 'mailer' with argument 'data'.
	proc << mailer << data.latin1();

	if ( !proc.start( KProcess::DontCare ) ) {
		KMessageBox::error(this,i18n("Could not start mailer."));
	}
}

//---------------------------------------------------------------------
// User has clicked on To Clipboard.

void MyldapKlient::slot_toClipboard() {
  QClipboard *cb = QApplication::clipboard();
	QListViewItem *selItem;
	int i;
  QString tag, data;

	// Shortcut. Really not needed, but nice.
	
	selItem = this->ldapView->selectedItem();

	// Scan through the columns untill we find the correct kabentrytag

	if ( selItem == NULL ) { return; };
	for (i= this->ldapView->columns(); i>0; i--) {
		tag = getKabentryTag(i-1);
		data = selItem->text(i-1);
		if (tag == "emails") 	{break;}
	}

  // The magic

  cb->setText(data);
}

//---------------------------------------------------------------------
// Popupmenu handler.

void MyldapKlient::slot_RMB( QListViewItem* Item, const QPoint & point, int )
{
  if( Item ) popUp->popup( point );
}
